//BePexesoApp.cpp

#include "BePexesoApp.h"

static long gameThread();

BePexesoApp::BePexesoApp()
	: BApplication("application/x-vnd.SerphWare-BePexeso")
{
	texturesFound = true; // used in ReadyToRun...
	BPath appPath;
	app_info appInfo;
	be_app->GetAppInfo(&appInfo);
	BEntry appEntry(&appInfo.ref, false);
	appEntry.GetPath(&appPath);
	strcpy(chExePath, appPath.Path());
	char *p = chExePath + strlen(chExePath);
	while( *p != '/' )
		p--;
	p[1] = '\0';
	
	if( LoadTilesets() == false)
	{
		texturesFound = false;
		(new BAlert("Error","No tilesets found\n Please download some tilesets and put them in the BePexeso folder.", "Oh crap..."))->Go();
		be_app->PostMessage(B_QUIT_REQUESTED);
	}
	
	LoadTextures();
	game = new BePexesoGame();
}

void
BePexesoApp::ReadyToRun()
{
	if(texturesFound)
	{
	mw = new BePexesoWin;
	mw->Show();
	gameTID = spawn_thread((thread_entry)&gameThread, "gameThread",B_NORMAL_PRIORITY, this);
	resume_thread(gameTID);
	}
}



void
BePexesoApp::AboutRequested()
{
	//(new BAlert("About BePexeso","BePexeso", "word..."))->Go();
  if( bPexesoMark ) /* make it non-reentrant */
    return;
  bPexesoMark = TRUE;

  bigtime_t dwEnd = real_time_clock_usecs();
  char str[256];

  sprintf(str, "Framerate is %f\n", dwFrames * 1000.0f/ (dwEnd-dwStart));
  printf(str);

  sprintf(str, "Framerate is %3.3f \nRenderer: %s %s \nVersion: %s", 
    dwFrames * 1000000.0f/ (dwEnd-dwStart),
    glVendor,
    glRenderer,
    glVersion
    );
    (new BAlert("About BePexeso",str , "word..."))->Go();

  //dwStart += real_time_clock_usecs() - dwEnd;

  bPexesoMark = FALSE;
}

bool
BePexesoApp::QuitRequested()
{
	return true;
}

bool
BePexesoApp::LoadTilesets(void)
{
	BDirectory appDir( chExePath );
	entry_ref appRef;
	int length;
	char* ext;
	nTilesets = 0;
	dwCurrentTS = -1;
	
	printf("Loading Tilesets...\n");
	
	while ((appDir.GetNextRef(&appRef) == B_OK) && (nTilesets < MAXTILESETS))
	{
		length = strlen(appRef.name);
		if ( length >= 4 )
		{
			ext = appRef.name + length - 3;
			if (!strcmp(ext, ".ts"))
			{
				printf("\t");
				printf(appRef.name);
				printf("\n");
				
				strcpy(chTilesets[nTilesets], appRef.name);
				char *p = chTilesets[nTilesets] + strlen(chTilesets[nTilesets]);
				while( *p != '.' )
					p--;
				*p = '\0';
				
				if( strcmp( chTilesets[nTilesets], chCurrentTS ) == 0 )
					dwCurrentTS = nTilesets;
				
				nTilesets ++;
			}
		}
	}
	
	printf("Done.\n\n");
	
	if( dwCurrentTS == -1 )
	{
		dwCurrentTS = 0;
		strcpy( chCurrentTS, chTilesets[dwCurrentTS] );
	}
	
	if ( nTilesets == 0 ){
		return false;
	}else{
		return true;
	}
}

void
BePexesoApp::LoadTextures()
{
//TILE_BACK_RGB
	FILE *f;
	
	char tmp[256];
	strcpy(tmp, chExePath);
	strcat(tmp, TILE_BACK_RGB);
	
	printf("Pathname:  %s\n", tmp); //debug
	f = fopen( tmp, "rb" );
	if ( f == NULL )
	{
		texturesFound = false;
		(new BAlert("Error","Could not find Tile Back texture.", "Oh crap..."))->Go();
		be_app->PostMessage(B_QUIT_REQUESTED);
	}	
	
	pTileTexture = new uchar[128*128*3];
	fseek(f, 32, SEEK_SET);
	fread(pTileTexture, 3, 128*128, f);
	fclose(f);
	
//BACKGROUND_RGB

	strcpy(tmp, chExePath);
	strcat(tmp, BACKGROUND_RGB);
	printf("Pathname:  %s\n", tmp); //debug
	f = fopen( tmp, "rb" );
	if ( f == NULL )
	{
		texturesFound = false;
		(new BAlert("Error","Could not find Background texture.", "Oh crap..."))->Go();
		be_app->PostMessage(B_QUIT_REQUESTED);
	}	
	
	pBackTexture = new uchar[256*256*3];
	fseek(f, 32, SEEK_SET);
	fread(pBackTexture, 3, 256*256, f);
	fclose(f);
}

BMenu*
BePexesoApp::GetTilesetsMenu()
{
	//Initialize the tileset menu
	tilesetMenu = new BMenu("Tileset", B_ITEMS_IN_COLUMN);
	tilesetMenu->SetRadioMode(true);
	for(int dw=0; dw<nTilesets; dw++)
	{
		tilesetMenu->AddItem(new BMenuItem( chTilesets[dw], new BMessage(MSG_CHANGE_TILESET)));
		if( dw == dwCurrentTS )
			( tilesetMenu->FindItem(chTilesets[dw]) )->SetMarked(true);
	}
	return(tilesetMenu);
}

int
main()
{
	BePexesoApp app;
	app.Run();
	return 0;
}

static long gameThread()
{
	((BePexesoApp *)be_app)->game->Run();
	return 1;
}