#include "BePexesoGame.h"
#include "BePexesoApp.h"

BePexesoGame::BePexesoGame()
{
	nTilesX = 5;
	nTilesY = 4;
}

void
BePexesoGame::Run()
{
	InitGame();
	
	bQuit = FALSE;
	bool bUpd;
	dwFrames = 0;
	((BePexesoApp *)be_app)->mw->mv->AllocTextureNames();
	((BePexesoApp *)be_app)->mw->mv->LoadTilesetTexture();
	((BePexesoApp *)be_app)->mw->mv->InitDisplay(xVP,yVP);
	dwStart = real_time_clock_usecs();
	bigtime_t dwNext = real_time_clock_usecs(), dwLast = dwNext;
	
	
	while(!bQuit)
	{
		bUpd = FALSE;
		dwElapsed = real_time_clock_usecs();
		if( dwElapsed >= dwNext )
		{
			((BePexesoApp *)be_app)->mw->mv->RenderFrame(); /* the redraw itself */
			((BePexesoApp *)be_app)->mw->mv->SwapBuffers(); /* double-buffered! */
			dwFrames ++;
			dwNext = dwElapsed + dwFrameLimit;
			bUpd = TRUE;
		}
		
		dwElapsed = real_time_clock_usecs();
		
		if( bUpd)
		{
			/*
			(dwElapsed-dwLast) is the time since the last UpdateState
			we pass it as a parameter, so that the animation speed can
			be adjusted accordingly
			*/
			UpdateState((dwElapsed - dwLast) / 1000000.0f);
			dwLast = dwElapsed;
		}
	}
}

void
BePexesoGame::InitGame()
{
  nTypes = 32;
  nOpen = 0;
  nOff = 0;
  int x,y;
  // reset geometric parameters
  for(y=0; y<nTilesY; y++) {
    for(x=0; x<nTilesX; x++) {
      tiles[y][x].fX = x * 1.0f - nTilesX / 2.0f + 0.5f;
      tiles[y][x].fY = y * 1.0f - nTilesY / 2.0f + 0.5f;
      tiles[y][x].xRot.x = 0.0f;
      tiles[y][x].xRot.v = 0.0f;
      tiles[y][x].yRot.x = 0.0f;
      tiles[y][x].yRot.v = 0.0f;
      tiles[y][x].nState = STATE_CLOSED;
      tiles[y][x].nNextState = STATE_CLOSED;
      tiles[y][x].nType = -1;
    }
  }

  // seed random number generator with system time
  srand(real_time_clock_usecs());

  // place pairs of random tiles all over the board
  // until all positions are filled
  
  int nTypeUsed[32]; // this will keep track of used types
  for(x=0; x<32; x++) {
    nTypeUsed[x] = 0;
  }

  int nPlaced = 0;

  while( nPlaced < nTilesY * nTilesX ) {
    int nType;

    // find a tile type which hasn't be placed yet
    do
      nType = rand() % nTypes;
    while( nTypeUsed[nType] );
    nTypeUsed[nType] = 1;

    int dwPos;

    // find a free tile position
    do 
      dwPos = rand() % (nTilesX*nTilesY);
    while( tiles[dwPos / nTilesX][dwPos % nTilesX].nType != -1);

    tiles[dwPos/nTilesX][dwPos%nTilesX].nType = nType;

    // find another free tile position
    do 
      dwPos = rand() % (nTilesX*nTilesY);
    while( tiles[dwPos / nTilesX][dwPos % nTilesX].nType != -1);

    tiles[dwPos/nTilesX][dwPos%nTilesX].nType = nType;

    nPlaced += 2;
  }
}


/* the tile at (x,y) has been clicked, process it */
void
BePexesoGame::Clicked(int x, int y)
{
  int i,j;
  switch(tiles[y][x].nState) {
  case STATE_CLOSED:
    /* closed? open it, possibly close others first */
    if( nOpen >= 2 ) {
      CheckMatch();
      for(i=0; i<nTilesY; i++)
        for(j=0; j<nTilesX; j++) {
          if(tiles[i][j].nState == STATE_OPEN) {
            tiles[i][j].fProgress = 0;
            tiles[i][j].nState = STATE_TRANS_CLOSE;
            tiles[i][j].nTransType = rand() % TRANS_CLOSE_S;
            nOpen --;
            Perturb(j,i,1.5f);
          } else if(tiles[i][j].nState == STATE_TRANS_OPEN) {
            if( tiles[i][j].nNextState != STATE_TRANS_OFF ) {
              tiles[i][j].nState = STATE_TRANS_CLOSE;
              tiles[i][j].fProgress = 1 - tiles[i][j].fProgress;
              nOpen--;
            }
          }
        }
    }
    tiles[y][x].fProgress = 0;
    tiles[y][x].nState = STATE_TRANS_OPEN;
    tiles[y][x].nTransType = rand() % TRANS_OPEN_S;
    nOpen ++;
    Perturb(x, y, 1.5f);
    break;
  case STATE_OPEN:
    /* open? pop */
    tiles[y][x].fProgress = 0;
    tiles[y][x].nState = STATE_TRANS_POP;
    tiles[y][x].nTransType = rand() % TRANS_POP_S;
    Perturb(x, y, 1.0f);
    break;
  case STATE_TRANS_OPEN:
  case STATE_TRANS_CLOSE:
    break;
  }
}

/* check for two matching open tiles */
/* quite an ugly implementation, I admit */
void
BePexesoGame::CheckMatch()
{
  int x,y;
  int xp[2], yp[2], tp[2];
  int i = 0;
  for(y=0; y<nTilesY; y++)
    for(x=0; x<nTilesX; x++) {
      if(tiles[y][x].nState == STATE_OPEN || 
         tiles[y][x].nState == STATE_TRANS_OPEN ||
         tiles[y][x].nState == STATE_TRANS_POP ) {
        xp[i] = x; yp[i] = y; tp[i] = tiles[y][x].nType;
        i++;
        assert(i<=2);
      }
    }
    if( i == 2 ) {
      if( tp[0] == tp[1] ) {
        if( tiles[yp[0]][xp[0]].nState == STATE_TRANS_OPEN )
          tiles[yp[0]][xp[0]].nNextState = STATE_TRANS_OFF;
        else {
          tiles[yp[0]][xp[0]].nState = STATE_TRANS_OFF;
          tiles[yp[0]][xp[0]].nTransType = rand() % TRANS_OFF_S;
          Perturb(xp[0], yp[0], -1.8f);
        }

        if( tiles[yp[1]][xp[1]].nState == STATE_TRANS_OPEN )
          tiles[yp[1]][xp[1]].nNextState = STATE_TRANS_OFF;
        else {
          tiles[yp[1]][xp[1]].nState = STATE_TRANS_OFF;
          tiles[yp[1]][xp[1]].nTransType = rand() % TRANS_OFF_S;
          Perturb(xp[1], yp[1], -1.8f);
        }
      }
    }
}

/* advance animation sequences, check for matching tiles etc. */
void
BePexesoGame::UpdateState(float time)
{

  int y, x;
  for(y=0; y<nTilesY; y++){
    for(x=0; x<nTilesX; x++) {
      switch(tiles[y][x].nState) {
      case STATE_TRANS_OFF:
        tiles[y][x].fProgress += time / 0.3f;
        if( tiles[y][x].fProgress > 1.0 ) {
          tiles[y][x].fProgress = 0;
          tiles[y][x].nState = STATE_OFF;
          nOpen --;
          nOff ++;
          if( nOff == nTilesX * nTilesY )
            InitGame();
        }
        break;
      case STATE_CLOSED:

        break;
      case STATE_OPEN:
        if( tiles[y][x].nNextState == STATE_TRANS_OFF ) {
          tiles[y][x].nState = STATE_TRANS_OFF;
          tiles[y][x].nTransType = rand() % TRANS_OFF_S;
          tiles[y][x].fProgress = 0;
          Perturb(x, y, -1.5);
        }
        break;
      case STATE_TRANS_OPEN:
        tiles[y][x].fProgress += time / 0.3f;
        if( tiles[y][x].fProgress > 1.0 ) {
          tiles[y][x].fProgress = 0;
          tiles[y][x].nState = STATE_OPEN;
          if( nOpen == 2 ) 
            CheckMatch();
        }
        break;
      case STATE_TRANS_CLOSE:
        tiles[y][x].fProgress += time / 0.3f;
        if( tiles[y][x].fProgress > 1.0 ) {
          tiles[y][x].fProgress = 0;
          tiles[y][x].nState = STATE_CLOSED;
        }
        break;
      case STATE_TRANS_POP:
        tiles[y][x].fProgress += time / 0.3f;
        if( tiles[y][x].fProgress > 1.0 ) {
          tiles[y][x].fProgress = 0;
          tiles[y][x].nState = STATE_OPEN;
        }
        break;
      }
    }
  }


  float fT = time / 1.0f; 

  /* this prevents problems on too slow machines (e.g. software renderers) */
  if( time > 0.3f || !bForeground ) {
    int x,y;
    for(y=0; y<nTilesY; y++) {
      for(x=0; x<nTilesX; x++) {
        tiles[y][x].xRot.x = 0.0f;
        tiles[y][x].xRot.v = 0.0f;
        tiles[y][x].yRot.x = 0.0f;
        tiles[y][x].yRot.v = 0.0f;
      }
    }
    fT = 0;
  }

  /* ugh, physics simulation */

  float fSpringHold = 0.7f; /* coeff for force holding tile in place */
  float fSpringMove = 0.9f; /* coeff for force of neighbors pulling tile away */
  float fM = 0.2f; /* mass */
  float fF = 0.0f; /* force */
  float fFriction = 0.97f; /* friction; gradual attenuation of speed */
  int nCnt;
  float fAvg;

  for(y=0; y<nTilesY; y++){
    for(x=0; x<nTilesX; x++) {
      nCnt = 0;
      fAvg = 0.0f;
      if( x > 0 ) { fAvg += tiles[y][x-1].xRot.x; nCnt++; }
      if( x < nTilesX-1 ) { fAvg += tiles[y][x+1].xRot.x; nCnt++; }
      if( y > 0 ) { fAvg += tiles[y-1][x].xRot.x; nCnt++; }
      if( y < nTilesY-1 ) { fAvg += tiles[y+1][x].xRot.x; nCnt++; }
      if( x > 0 && y > 0 ) { fAvg += tiles[y-1][x-1].xRot.x; nCnt++; }
      if( x > 0 && y < nTilesY-1 ) { fAvg += tiles[y+1][x-1].xRot.x; nCnt++; }
      if( x < nTilesX-1 && y > 0 ) { fAvg += tiles[y-1][x+1].xRot.x; nCnt++; }
      if( x < nTilesX-1 && y < nTilesY-1 ) { fAvg += tiles[y+1][x+1].xRot.x; nCnt++; }

      assert(nCnt);
      fAvg /= nCnt;
  
      fF = (fAvg - tiles[y][x].xRot.x) * fSpringMove - tiles[y][x].xRot.x * fSpringHold;
      tiles[y][x].xNextRot.x = tiles[y][x].xRot.x + tiles[y][x].xRot.v * fT;
      tiles[y][x].xNextRot.v = (tiles[y][x].xRot.v + fF * fT / fM) * fFriction;
      
      nCnt = 0;
      fAvg = 0.0f;
      if( x > 0 ) { fAvg += tiles[y][x-1].xRot.x; nCnt++; }
      if( x < nTilesX-1 ) { fAvg += tiles[y][x+1].xRot.x; nCnt++; }
      if( y > 0 ) { fAvg += tiles[y-1][x].yRot.x; nCnt++; }
      if( y < nTilesY-1 ) { fAvg += tiles[y+1][x].yRot.x; nCnt++; }
      if( x > 0 && y > 0 ) { fAvg += tiles[y-1][x-1].yRot.x; nCnt++; }
      if( x > 0 && y < nTilesY-1 ) { fAvg += tiles[y+1][x-1].yRot.x; nCnt++; }
      if( x < nTilesX-1 && y > 0 ) { fAvg += tiles[y-1][x+1].yRot.x; nCnt++; }
      if( x < nTilesX-1 && y < nTilesY-1 ) { fAvg += tiles[y+1][x+1].yRot.x; nCnt++; }

      assert(nCnt);
      fAvg /= nCnt;
  
      fF = (fAvg - tiles[y][x].yRot.x) * fSpringMove - tiles[y][x].yRot.x * fSpringHold;
      tiles[y][x].yNextRot.x = tiles[y][x].yRot.x + tiles[y][x].yRot.v * fT;
      tiles[y][x].yNextRot.v = (tiles[y][x].yRot.v + fF * fT / fM) * fFriction;
    }
  }

  for(y=0; y<nTilesY; y++){
    for(x=0; x<nTilesX; x++) {
      tiles[y][x].xRot = tiles[y][x].xNextRot;
      tiles[y][x].yRot = tiles[y][x].yNextRot;
    }
  }
  
}

#define PERTURB(a,b) a=a*0.8f+(b)

void
BePexesoGame::Perturb(int x, int y, float fAmp)
{
  if( !bWaves ) return;
    if( x > 0 ) 
      PERTURB(tiles[y][x-1].xRot.v, -fAmp);
    if( x < nTilesX-1 ) 
      PERTURB(tiles[y][x+1].xRot.v, fAmp);
    if( y > 0 ) 
      PERTURB(tiles[y-1][x].yRot.v, fAmp);
    if( y < nTilesY-1 ) 
      PERTURB(tiles[y+1][x].yRot.v, -fAmp);

    if( x > 0 && y > 0 ) {
      PERTURB(tiles[y-1][x-1].xRot.v, -fAmp*0.63f);
      PERTURB(tiles[y-1][x-1].yRot.v, fAmp*0.63f);
    }
    if( x > 0 && y < nTilesY-1 ) {
      PERTURB(tiles[y+1][x-1].xRot.v, -fAmp*0.63f);
      PERTURB(tiles[y+1][x-1].yRot.v, -fAmp*0.63f);
    }
    if( x < nTilesX-1 && y > 0 ) {
      PERTURB(tiles[y-1][x+1].xRot.v, fAmp*0.63f);
      PERTURB(tiles[y-1][x+1].yRot.v, fAmp*0.63f);
    }
    if( x < nTilesX-1 && y < nTilesY-1 ) {
      PERTURB(tiles[y+1][x+1].xRot.v, fAmp*0.63f);
      PERTURB(tiles[y+1][x+1].yRot.v, -fAmp*0.63f);
    }
}
