//BePexesoView.cpp

#include "BePexesoView.h"
#include "BePexesoApp.h"

BePexesoView::BePexesoView(BRect R)
	: BGLView(R, "BePexesoview", B_FOLLOW_ALL, 0, BGL_RGB | BGL_DEPTH | BGL_DOUBLE )
{
    xVP = (R.right - R.left);
    yVP = (R.bottom - R.top);
    LockGL();
    glVendor = (char* )glGetString(GL_VENDOR);
    glRenderer = (char* )glGetString(GL_RENDERER);
    glVersion = (char* )glGetString(GL_VERSION);
    UnlockGL();
}

BePexesoView::~BePexesoView()
{
	// empty
}

void
BePexesoView::InitDisplay(int width, int height)
{
	// calculate texture centers
	xVP = width;
	yVP = height;
	int i,j;
	for(i=0; i<8; i++)
	{
		for(j=0; j<4; j++)
		{
			fTexCent[8*j+i][0] = 1/16.0f + i/8.0f;
			fTexCent[8*j+i][1] = 1/8.0f + j/4.0f;
		}
	}
	
	LockGL();
	// set viewport
	glViewport(0, 0, width, height);
	glGetIntegerv(GL_VIEWPORT, viewport);

	// set orthographic (non-perspective) projection
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	
	glOrtho(-nTilesX/2.0f, nTilesX/2.0f, -nTilesY/2.0f, nTilesY/2.0f, 4.90f, 10.0f);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_DEPTH_TEST);
	glClearDepth(1.0);
	glDepthFunc(GL_LEQUAL);  /* Superimpose smaller Z values over larger ones */

	// material for tiles - not too shiny
	GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
	GLfloat mat_shininess[] = { 1.0 };
	
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

	GLfloat light0_amb[] = { 0.6f, 0.6f, 0.6f, 1.0f };
	GLfloat light0_dif[] = { 0.6f, 0.6f, 0.6f, 1.0f };
	GLfloat light0_dif_nl[] = { 0.9f, 0.9f, 0.9f, 1.0f }; // no spotlight
	GLfloat light0_spc[] = { 0.0f, 0.0f, 0.0f, 1.0f };
	GLfloat light0_pos[] = { 0.0f, 0.0f, 1.0f, 0.0f };

	/*
	second and third light sources form the spotlight -
	a more intensive one on the inside and a weaker one
	with a wider angle on the outside
	*/
	if( bSpotlight ) {

	glLightfv(GL_LIGHT0, GL_AMBIENT, light0_amb);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_dif);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light0_spc);
    glLightfv(GL_LIGHT0, GL_POSITION, light0_pos);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);

    GLfloat light1_amb[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat light1_dif[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat light1_spc[] = { 1.0f, 1.0f, 0.8f, 1.0f };
    GLfloat light1_pos[] = { 0.0f, 0.0f, 1.0f, 1.0f };

    glLightfv(GL_LIGHT1, GL_AMBIENT, light1_amb);
    glLightfv(GL_LIGHT1, GL_DIFFUSE, light1_dif);
    glLightfv(GL_LIGHT1, GL_SPECULAR, light1_spc);
    glLightfv(GL_LIGHT1, GL_POSITION, light1_pos);
  
    glLightf (GL_LIGHT1, GL_SPOT_CUTOFF, 6.0);
    GLfloat spot_direction[] = { 0.0, 0.0, DEPTH };

    glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, spot_direction);

    glEnable(GL_LIGHT1);

    GLfloat light2_amb[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat light2_dif[] = { 0.0f, 0.0f, 0.0f, 1.0f };
    GLfloat light2_spc[] = { 0.15f, 0.15f, 0.15f, 1.0f };
    GLfloat light2_pos[] = { 0.0f, 0.0f, 1.0f, 1.0f };

    glLightfv(GL_LIGHT2, GL_AMBIENT, light2_amb);
    glLightfv(GL_LIGHT2, GL_DIFFUSE, light2_dif);
    glLightfv(GL_LIGHT2, GL_SPECULAR, light2_spc);
    glLightfv(GL_LIGHT2, GL_POSITION, light2_pos);
  
    glLightf (GL_LIGHT2, GL_SPOT_CUTOFF, 11.0);

    glLightfv(GL_LIGHT2, GL_SPOT_DIRECTION, spot_direction);

    glEnable(GL_LIGHT2);
  } else {
    /* no spotlight, only the diffuse/ambient light */

    glLightfv(GL_LIGHT0, GL_AMBIENT, light0_dif_nl);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light0_dif_nl);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light0_spc);
    glLightfv(GL_LIGHT0, GL_POSITION, light0_pos);

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glDisable(GL_LIGHT1);
    glDisable(GL_LIGHT2);

  }

 /* enable backface culling, this speed things up a lot */
 	glEnable(GL_CULL_FACE);
 	
 	/* enable texturing */
  glEnable(GL_TEXTURE_2D);

  /* load tile texture */
  glBindTexture(GL_TEXTURE_2D, texNames[TILETEX]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, 3, 128, 128, 0, GL_RGB, GL_UNSIGNED_BYTE, pTileTexture);

  /* load background texture */
  glBindTexture(GL_TEXTURE_2D, texNames[BACKTEX]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, 3, 256, 256, 0, GL_RGB, GL_UNSIGNED_BYTE, pBackTexture);

  // Create display list for back of tile
  nTileBackList = glGenLists(1);

  glNewList(nTileBackList, GL_COMPILE);
  DrawTileBack();
  glEndList();
  UnlockGL();
}

void
BePexesoView::RenderFrame(void )
{
  int x,y;
  LockGL();
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  /* clear buffers */
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  GLfloat spot_direction[] = { 0.0, 0.0, DEPTH };
  spot_direction[0] = (((GLfloat)xMousePos)/viewport[2]-0.5f)*nTilesX;
  spot_direction[1] = (((GLfloat)viewport[3]-yMousePos)/viewport[3]-0.5f)*nTilesY;

  /* set direction of spotlight light sources */
  glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, spot_direction);
  glLightfv(GL_LIGHT2, GL_SPOT_DIRECTION, spot_direction);
  
  /* render tiles */
  for(y=0; y<nTilesY; y++)
    for(x=0; x<nTilesX; x++) {
      RenderTile(&(tiles[y][x]));
    }
  
  /* render background */
  RenderBackground();

  glFlush();
  UnlockGL();
}


/* find which tile falls under pixel (scrx, scry)
   use OpenGL selection mechanism
   basically, same code as the drawing */
int
BePexesoView::PickTile(int scrx, int scry, int& rx, int& ry)
{
	LockGL();
  int x,y;
  GLuint sel[16];
  
  glSelectBuffer(16, sel);

  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();

  /*
      create a projection matrix which allows rendering (hitting)
      only of a 5x5 pixel square around the mouse cursor 
  */
     
  gluPickMatrix( (GLdouble)scrx, (GLdouble)(viewport[3]-scry), 5, 5, viewport);
/*  glFrustum(-nTilesX/2.0f, nTilesX/2.0f, 
            -nTilesY/2.0f, nTilesY/2.0f, 
            4.90f, 15.0f);*/
  glOrtho(-nTilesX/2.0f, nTilesX/2.0f, 
            -nTilesY/2.0f, nTilesY/2.0f, 
            4.90f, 10.0f);
  glRenderMode(GL_SELECT);
  glInitNames();
  glPushName(1);
  
  glMatrixMode(GL_MODELVIEW);

  for(y=0; y<nTilesY; y++)
    for(x=0; x<nTilesX; x++) {
      glLoadName(y*256+x);
      RenderTile(&(tiles[y][x]));
    }

  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glFlush();
  
  rx = ry = -1;

  int nHits = glRenderMode(GL_RENDER);
UnlockGL();
  if( nHits > 0 ) {
    ry = sel[3] / 256;
    rx = sel[3] % 256;
    return 1;
  } else {
    return 0;
  }

}

void
BePexesoView::RenderTile(Tile *pTile)
{
  if( pTile->nState == STATE_OFF )
    return;

  glMatrixMode(GL_MODELVIEW);

  glLoadIdentity();
  /*
  Translate tile into place
  */

  glTranslatef(pTile->fX, pTile->fY, DEPTH );

  /*
    Rotate tile
  */

#define CLAMP_ANGLE(fi) ((float)(fi<-90.0f?-90.f:(fi>90.0f?90.0f:fi)))
#define abs(x) ((x)<0?-(x):(x))
#define hypot(x,y) sqrt(x*x+y*y)

  /*
    position of tile on wavy surface
  */
  if( bWaves ) {
    if( pTile->nState == STATE_CLOSED || pTile->nState == STATE_OPEN ) {

      glRotatef(CLAMP_ANGLE(hypot(pTile->xRot.x, pTile->yRot.x)*40.0f),
        pTile->yRot.x, pTile->xRot.x, 0.0f);

      glTranslatef(pTile->xRot.x * 0.12f, pTile->yRot.x * 0.12f, 0.0f);
    } 
  }

  float fProgress = pTile->fProgress;
  float fPar = (-fProgress*fProgress + fProgress); // for quadratic animations, like the pop

  /*
    rotations due to animations in progress
  */
  switch(pTile->nState) {
  case STATE_OPEN:
    break;
  case STATE_CLOSED:
    glRotatef(180.0f, 0.0f, 1.0f, 0.0f);
    break;
  case STATE_TRANS_OPEN:
    switch( pTile->nTransType ) {
    case 0: glRotatef(180.0f*(1.0f-fProgress), 0.0f, 1.0f, 0.0f); break;
    case 1: glRotatef(-180.0f*(1.0f-fProgress), 0.0f, 1.0f, 0.0f); break;
    case 2: glRotatef(180.0f*(1.0f-fProgress), fPar, 1.0f, 0.0f); break;
    case 3: glRotatef(-180.0f*(1.0f-fProgress), fPar, 1.0f, 0.0f); break;
    case 4: glRotatef(180.0f*(1.0f-fProgress), 0.0f, 1.0f, fPar); break;
    case 5: glRotatef(-180.0f*(1.0f-fProgress), 0.0f, 1.0f, fPar); break;
    case 6: glRotatef(180.0f*(1.0f-fProgress), fPar, 1.0f, fPar); break;
    case 7: glRotatef(-180.0f*(1.0f-fProgress), fPar, 1.0f, fPar); break;
    }
    break;
  case STATE_TRANS_CLOSE:
    switch( pTile->nTransType ) {
    case 0: glRotatef(180.0f*fProgress, 0.0f, 1.0f, fPar); break;
    case 1: glRotatef(-180.0f*fProgress, 0.0f, 1.0f, fPar); break;
    case 2: glRotatef(180.0f*fProgress, fPar, 1.0f, 0.0f); break;
    case 3: glRotatef(-180.0f*fProgress, fPar, 1.0f, 0.0f); break;
    case 4: glRotatef(180.0f*fProgress, 0.0f, 1.0f, 0.0f); break;
    case 5: glRotatef(-180.0f*fProgress, 0.0f, 1.0f, 0.0f); break;
    }
    break;
  case STATE_TRANS_POP:
    {
      switch( pTile->nTransType ){
      case 0:
        glScalef(1+fPar*5, 1+fPar*5, 1);//1+fPar*3, 1);
        glTranslatef(0.0f, 0.0f, 3.0f*fPar);
        break;
      case 1:
        glScalef(1+fPar*4, 1+fPar*4, 1);
        glTranslatef(0.0f, 0.0f, 3.0f*fPar);
        glRotatef(fPar*4*90.0f, 0.0f, 0.0f, 1.0f);
        break;
      case 2:
        glScalef(1+fPar*4, 1+fPar*4, 1);
        glTranslatef(0.0f, 0.0f, 3.0f*fPar);
        glRotatef(-fPar*4*90.0f, 0.0f, 0.0f, 1.0f);
        break;
      }
    }
    break;
  case STATE_TRANS_OFF:
    switch( pTile->nTransType ) {
    case 0:
      glScalef(1-fProgress, 1-fProgress, 1);
      glRotatef(fProgress * 180.0f, 1, 1, 1);
      break;
    case 1:
      glScalef(1-fProgress, 1-fProgress, 1);
      glRotatef(fProgress * 180.0f, 1, 1, 0);
      break;
    case 2:
      glScalef(1-fProgress, 1-fProgress, 1);
      glRotatef(fProgress * 180.0f, 1, 0, 1);
      break;
    case 3:
      glScalef(1-fProgress, 1-fProgress, 1);
      glRotatef(fProgress * 180.0f, 0, 1, 1);
      break;
    case 4:
      glScalef(1-fProgress/2, 1-fProgress, 1);
      glRotatef(fProgress * 180.0f, 1, 1, 1);
      break;
    case 5:
      glScalef(1-fProgress/2, 1-fProgress, 1);
      glRotatef(fProgress * 180.0f, 1, 1, 0);
      break;
    case 6:
      glScalef(1-fProgress, 1-fProgress/2, 1);
      glRotatef(fProgress * 180.0f, 1, 0, 1);
      break;
    case 7:
      glScalef(1-fProgress, 1-fProgress/2, 1);
      glRotatef(fProgress * 180.0f, 0, 1, 1);
      break;
    }
  }

  glCallList(nTileBackList); // oddly, this doesn't bring _any_ speedup ???
  DrawTileFace(pTile->nType);
}

/*
  a weird way of specifying geometry - but easier for our purposes than
  importing model files
*/
#define XA  0.37f
#define XB  0.07f
#define YA  0.18f

#define TA  0.95f/8

#define FMM glTexCoord2f(fTexCentX, fTexCentY);       glVertex3f(0, 0, YA)   
#define FTL glTexCoord2f(fTexCentX-0.5f*TA, fTexCentY+TA); glVertex3f(-XA, XA, YA)   
#define FTM glTexCoord2f(fTexCentX, fTexCentY+TA);    glVertex3f(0, XA, YA)   
#define FTR glTexCoord2f(fTexCentX+0.5f*TA, fTexCentY+TA); glVertex3f(XA, XA, YA)    
#define FRM glTexCoord2f(fTexCentX+0.5f*TA, fTexCentY);    glVertex3f(XA, 0, YA)   
#define FBR glTexCoord2f(fTexCentX+0.5f*TA, fTexCentY-TA); glVertex3f(XA, -XA, YA)   
#define FBM glTexCoord2f(fTexCentX, fTexCentY-TA);    glVertex3f(0, -XA, YA)   
#define FLM glTexCoord2f(fTexCentX-0.5f*TA, fTexCentY);    glVertex3f(-XA, 0, YA)   
#define FBL glTexCoord2f(fTexCentX-0.5f*TA, fTexCentY-TA); glVertex3f(-XA, -XA, YA)

void
BePexesoView::DrawTileFace(int nType)
{
  float fTexCentX, fTexCentY;

  fTexCentX = fTexCent[nType][0];
  fTexCentY = fTexCent[nType][1];

  glColor3f(0.0f, 0.0f, 0.0f);
  glNormal3f(0.0f, 0.0f, 1.0f);

  glBindTexture(GL_TEXTURE_2D, texNames[TILESET]);
  glEnable(GL_BLEND); // enable blending to superimpose picture on tile face
  glBlendFunc(GL_SRC_ALPHA, GL_SRC_ALPHA);
  glBegin(GL_TRIANGLE_FAN);

  FMM; FRM;
  FTR; FTM; FTL; FLM;
  FBL; FBM; FBR; FRM;

  glEnd();
  glDisable(GL_BLEND);
}
// hack
#undef TA 
#undef TB 

#undef FTL    
#undef FTR  
#undef FBR   
#undef FBL    

#undef FMM
#undef FTM
#undef FRM
#undef FBM
#undef FLM

#undef MTLA
#undef MTRA
#undef MBRA
#undef MBLA

#undef MTM 
#undef MBM 
#undef MLM 
#undef MRM 

#undef MTLB 
#undef MTRB 
#undef MBRB 
#undef MBLB 

#undef SQ

#undef MTLC
#undef MTRC
#undef MBRC 
#undef MBLC 

#undef BTL
#undef BTR
#undef BBR 
#undef BBL 

#undef BMM 
#undef BTM 
#undef BRM 
#undef BBM 
#undef BLM 



// defines
#define TA  0.2f
#define TB  0.1f

#define FTL glNormal3f(0.0f, 0.0f, 1.0f); glTexCoord2f(0.5f-TA, 0.5f+TA); glVertex3f(-XA, XA, YA)   
#define FTR glNormal3f(0.0f, 0.0f, 1.0f); glTexCoord2f(0.5f+TA, 0.5f+TA); glVertex3f(XA, XA, YA)    
#define FBR glNormal3f(0.0f, 0.0f, 1.0f); glTexCoord2f(0.5f+TA, 0.5f-TA); glVertex3f(XA, -XA, YA)   
#define FBL glNormal3f(0.0f, 0.0f, 1.0f); glTexCoord2f(0.5f-TA, 0.5f-TA); glVertex3f(-XA, -XA, YA)   

#define FMM glNormal3f(0.0f, 0.0f, 1.0f); glTexCoord2f(0.5f, 0.5f);       glVertex3f(0, 0, YA)   
#define FTM glNormal3f(0.0f, 0.0f, 1.0f); glTexCoord2f(0.5f, 0.5f+TA);    glVertex3f(0, XA, YA)   
#define FRM glNormal3f(0.0f, 0.0f, 1.0f); glTexCoord2f(0.5f+TA, 0.5f);    glVertex3f(XA, 0, YA)   
#define FBM glNormal3f(0.0f, 0.0f, 1.0f); glTexCoord2f(0.5f, 0.5f-TA);    glVertex3f(0, -XA, YA)   
#define FLM glNormal3f(0.0f, 0.0f, 1.0f); glTexCoord2f(0.5f-TA, 0.5f);    glVertex3f(-XA, 0, YA)   

#define MTLA glNormal3f(0.0f, 1.0f, 0.0f); glTexCoord2f(0.5f-TA, 0.5f+TA+TB)  ;  glVertex3f(-(XA), (XA+XB), 0)   
#define MTRA glNormal3f(1.0f, 0.0f, 0.0f); glTexCoord2f(0.5f+TA+TB, 0.5f+TA)  ;  glVertex3f((XA+XB), (XA), 0)    
#define MBRA glNormal3f(0.0f, -1.0f, 0.0f); glTexCoord2f(0.5f+TA, 0.5f-(TA+TB));  glVertex3f((XA), -(XA+XB), 0)   
#define MBLA glNormal3f(-1.0f, 0.0f, 0.0f); glTexCoord2f(0.5f-(TA+TB), 0.5f-TA);  glVertex3f(-(XA+XB), -(XA), 0)   

#define MTM glNormal3f(0.0f, 1.0f, 0.0f); glTexCoord2f(0.5f, 0.5f+TA+TB); glVertex3f(0, (XA+XB), 0)                      
#define MBM glNormal3f(0.0f, -1.0f, 0.0f); glTexCoord2f(0.5f, 0.5f-(TA+TB)); glVertex3f(0, -(XA+XB), 0)                      
#define MLM glNormal3f(-1.0f, 0.0f, 0.0f); glTexCoord2f(0.5f-(TA+TB), 0.5f); glVertex3f(-(XA+XB), 0, 0)                      
#define MRM glNormal3f(1.0f, 0.0f, 0.0f); glTexCoord2f(0.5f+(TA+TB), 0.5f); glVertex3f((XA+XB), 0, 0)                      

#define MTLB glNormal3f(-1.0f, 0.0f, 0.0f); glTexCoord2f(0.5f-(TA+TB), 0.5f+TA) ; glVertex3f(-(XA+XB), (XA), 0)   
#define MTRB glNormal3f(0.0f, 1.0f, 0.0f);  glTexCoord2f(0.5f+TA, 0.5f+(TA+TB)) ; glVertex3f((XA), (XA+XB), 0)    
#define MBRB glNormal3f(1.0f, 0.0f, 0.0f);  glTexCoord2f(0.5f+(TA+TB), 0.5f-TA) ; glVertex3f((XA+XB), -(XA), 0)   
#define MBLB glNormal3f(0.0f, -1.0f, 0.0f); glTexCoord2f(0.5f-TA, 0.5f-(TA+TB)); glVertex3f(-(XA), -(XA+XB), 0)  

#define SQ  0.7071f

#define MTLC glNormal3f(-1.0f, 1.0f, 0.0f); glTexCoord2f(0.5f-(TA+TB*SQ), 0.5f+(TA+TB*SQ)) ; glVertex3f(-(XA+XB*SQ),  (XA+XB*SQ), 0)   
#define MTRC glNormal3f(1.0f, 1.0f, 0.0f); glTexCoord2f(0.5f+(TA+TB*SQ), 0.5f+(TA+TB*SQ)) ; glVertex3f( (XA+XB*SQ),  (XA+XB*SQ), 0)    
#define MBRC glNormal3f(1.0f, -1.0f, 0.0f); glTexCoord2f(0.5f+(TA+TB*SQ), 0.5f-(TA+TB*SQ)) ; glVertex3f( (XA+XB*SQ), -(XA+XB*SQ), 0)   
#define MBLC glNormal3f(-1.0f, -1.0f, 0.0f); glTexCoord2f(0.5f-(TA+TB*SQ), 0.5f-(TA+TB*SQ)) ; glVertex3f(-(XA+XB*SQ), -(XA+XB*SQ), 0)  

#define BTL glNormal3f(0.0f, 0.0f, -1.0f); glTexCoord2f(0.5f-TA, 0.5f+TA) ; glVertex3f(-XA, XA, -YA)
#define BTR glNormal3f(0.0f, 0.0f, -1.0f); glTexCoord2f(0.5f+TA, 0.5f+TA) ; glVertex3f(XA, XA, -YA)    
#define BBR glNormal3f(0.0f, 0.0f, -1.0f); glTexCoord2f(0.5f+TA, 0.5f-TA) ; glVertex3f(XA, -XA, -YA)   
#define BBL glNormal3f(0.0f, 0.0f, -1.0f); glTexCoord2f(0.5f-TA, 0.5f-TA) ; glVertex3f(-XA, -XA, -YA)  

#define BMM glNormal3f(0.0f, 0.0f, -1.0f); glTexCoord2f(0.5f, 0.5f);       glVertex3f(0, 0, -YA)   
#define BTM glNormal3f(0.0f, 0.0f, -1.0f); glTexCoord2f(0.5f, 0.5f+TA);    glVertex3f(0, XA, -YA)   
#define BRM glNormal3f(0.0f, 0.0f, -1.0f); glTexCoord2f(0.5f+TA, 0.5f);    glVertex3f(XA, 0, -YA)   
#define BBM glNormal3f(0.0f, 0.0f, -1.0f); glTexCoord2f(0.5f, 0.5f-TA);    glVertex3f(0, -XA, -YA)   
#define BLM glNormal3f(0.0f, 0.0f, -1.0f); glTexCoord2f(0.5f-TA, 0.5f);    glVertex3f(-XA, 0, -YA)

void
BePexesoView::DrawTileBack()
{
  //changed from 0
  glColor3f(1.0f, 1.0f, 1.0f);

  glBindTexture(GL_TEXTURE_2D, texNames[TILETEX]);
  glBegin(GL_QUADS);

  FTR; MTRB; MTM; FTM; FTM; MTM; MTLA; FTL;
  FTL; MTLB; MLM; FLM; FLM; MLM; MBLA; FBL;
  FBL; MBLB; MBM; FBM; FBM; MBM; MBRA; FBR;
  FBR; MBRB; MRM; FRM; FRM; MRM; MTRA; FTR;
  
  MTRB; BTR; BTM; MTM; MTM; BTM; BTL; MTLA;
  MTLB; BTL; BLM; MLM; MLM; BLM; BBL; MBLA;
  MBLB; BBL; BBM; MBM; MBM; BBM; BBR; MBRA;
  MBRB; BBR; BRM; MRM; MRM; BRM; BTR; MTRA;

  glEnd();

  glBegin(GL_TRIANGLE_FAN);

  BMM; BLM;
  BTL; BTM; BTR; BRM;
  BBR; BBM; BBL; BLM;

  glEnd();

  glBegin(GL_TRIANGLE_FAN);

  FMM; FRM;
  FTR; FTM; FTL; FLM;
  FBL; FBM; FBR; FRM;

  glEnd();

  glBegin(GL_TRIANGLES);

  FTR; MTRA; MTRC;
  FTR; MTRC; MTRB;
  BTR; MTRB; MTRC;
  BTR; MTRC; MTRA;

  FTL; MTLA; MTLC;
  FTL; MTLC; MTLB;
  BTL; MTLB; MTLC;
  BTL; MTLC; MTLA;

  FBR; MBRA; MBRC;
  FBR; MBRC; MBRB;
  BBR; MBRB; MBRC;
  BBR; MBRC; MBRA;

  FBL; MBLA; MBLC;
  FBL; MBLC; MBLB;
  BBL; MBLB; MBLC;
  BBL; MBLC; MBLA;

  glEnd();
}

#define TEXBEGINX 0.0f
#define TEXSIZEX   1.0f
#define TEXBEGINY 0.0f
#define TEXSIZEY   1.0f

void
BePexesoView::RenderBackground()
{
  glMatrixMode(GL_MODELVIEW);

  glLoadIdentity();

  glNormal3f(0.0f, 0.0f, 1.0f);
  glTranslatef(0, 0, DEPTH - 2.0f);

  int i,j;
  glBindTexture(GL_TEXTURE_2D, texNames[BACKTEX]);
  glBegin(GL_TRIANGLES);

  if( bWaves ) {
#define TILEVERTEX(i, j) \
  glTexCoord2f( TEXBEGINX+(i+1)*(TEXSIZEX)/(nTilesX+2), TEXBEGINY+(j+1)*(TEXSIZEY)/(nTilesY+2)); \
  glVertex3f( i * 1.0f - nTilesX / 2.0f + 0.5f + ((i>0 && i<nTilesX-1)?tiles[j][i].xRot.x*0.3f:0.0f), \
  j * 1.0f - nTilesX / 2.0f + 0.5f + ((j>0 && j<nTilesY-1)?tiles[j][i].yRot.x*0.3f:0.0f), \
              0.0f)

    for(j=-1; j<nTilesY; j++) {
      for(i=-1; i<nTilesX; i++) {
        TILEVERTEX(i, j);
        TILEVERTEX(i+1, j);
        TILEVERTEX(i+1, j+1);

        TILEVERTEX(i, j);
        TILEVERTEX(i+1, j+1);
        TILEVERTEX(i, j+1);
      }
    }
  } else {
#undef TILEVERTEX
#define TILEVERTEX(i, j) \
  glTexCoord2f( TEXBEGINX+(i+1)*(TEXSIZEX)/(nTilesX+2), TEXBEGINY+(j+1)*(TEXSIZEY)/(nTilesY+2)); \
  glVertex3f( i * 1.0f - nTilesX / 2.0f + 0.5f, \
  j * 1.0f - nTilesX / 2.0f + 0.5f, \
              0.0f)

    for(j=-1; j<nTilesY; j++) {
      for(i=-1; i<nTilesX; i++) {
        TILEVERTEX(i, j);
        TILEVERTEX(i+1, j);
        TILEVERTEX(i+1, j+1);

        TILEVERTEX(i, j);
        TILEVERTEX(i+1, j+1);
        TILEVERTEX(i, j+1);
      }
    }
  }

  glEnd();
  glLoadIdentity();
}

/*
  load the texture for the current tileset
*/
void
BePexesoView::LoadTilesetTexture(void)
{
	char tmp[256];
	
	strcpy(tmp, chExePath);
	strcat(tmp, chCurrentTS);
	strcat(tmp, ".ts");
	printf(tmp);
	printf("\n");
	
	if( pTileset )
		delete pTileset;
		
	pTileset = new uchar[512 * 256];

	FILE *f;
	f = fopen( tmp, "rb" );
	if ( f == NULL )
	{
		(new BAlert("Error","Could not open current tileset.", "Oh crap..."))->Go();
		be_app->PostMessage(B_QUIT_REQUESTED);
	}
	
	fseek(f, 32, SEEK_SET);
	fread(pTileset, 1, 512 * 256, f);
	fclose(f);

  uchar *pTilesetAlpha = new uchar[512*256*4];

  /*
    expand grayscale to RGBA; don't let the alpha reach 0 so the
    picture looks slightly transparent on the tile surface
  */
  for(int dw=0; dw<512*256; dw++) {
    pTilesetAlpha[dw*4] = 0;
    pTilesetAlpha[dw*4+1] = 0;
    pTilesetAlpha[dw*4+2] = 0;
    pTilesetAlpha[dw*4+3] = (unsigned char)(255-(255-pTileset[dw])/1.4f);
  }
  
  	delete pTileset;
  	pTileset = pTilesetAlpha;
	LockGL();
	glBindTexture(GL_TEXTURE_2D, texNames[TILESET]);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  	glTexImage2D(GL_TEXTURE_2D, 0, 4, 512, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, pTileset);
	UnlockGL();
}

void
BePexesoView::MouseDown(BPoint where)
{
    PickTile(where.x, where.y, nXTileDown, nYTileDown);
    if( nXTileDown != -1 )
    	((BePexesoApp *)be_app)->game->Clicked(nXTileDown, nYTileDown);
}

void
BePexesoView::MouseMoved(BPoint point, uint32 transit, const BMessage *message)
{
	if (transit == B_INSIDE_VIEW )
	{
		xMousePos = point.x;
		yMousePos = point.y;
	}
}

void
BePexesoView::AllocTextureNames(void )
{
	LockGL();
	glGenTextures(3, texNames);
	UnlockGL();
}
